/**
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $id.java
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.engine.kpi;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.UtilFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.wsstar.addressing.definition.WSAddressingFactory;
import com.ebmwebsourcing.wsstar.addressing.definition.api.EndpointReferenceType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.ReferenceParametersType;
import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.WSNotificationFactory;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.WsnbConstants;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.FilterType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.QueryExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Subscribe;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscribeResponse;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.SubscriptionPolicyType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.TopicExpressionType;
import com.ebmwebsourcing.wsstar.notification.definition.basenotification.api.Unsubscribe;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationReader;
import com.ebmwebsourcing.wsstar.notification.definition.inout.WSNotificationWriter;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.WSNotificationExtensionFactory;
import com.ebmwebsourcing.wsstar.notification.extension.api.ContextPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ProcessPolicyType;
import com.ebmwebsourcing.wsstar.notification.extension.api.ResourcesUuidType;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.api.XsltStyleSheetType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WsnSpecificTypeHelper;
import com.ebmwebsourcing.wsstar.notification.jaxbextension.types.ActionType;
import com.ebmwebsourcing.wsstar.notification.service.util.WSNotificationNotImplementedException;

public class KeyProcessIndicatorService {

    private File receivedUuidParentFolder;

    // forward notification request to NotificationBrokerEngine :
    private static final QName NOTIFICATION_BROKER_SERVICE_QNAME = new QName(
            "http://petals.ow2.org/petals-se-notification", "NotificationBrokerService");

    private static final QName NOTIFICATION_BROKER_INTERFACE_QNAME = new QName(
            "http://docs.oasis-open.org/wsn/brw-2", "NotificationBroker");

    private DeliveryChannel dc = null;

    private ComponentContext cptContext = null;

    private SOAParameterType consumerEndpoint = null;

    // List<String> susbcriptionsProcessedOnInit = null;
    private Map<String, EndpointReferenceType> returnedSubscriptionRefs = null;

    private Logger logger = null;

    private String[] additionnalNsAndPrefixMapping = null;

    public KeyProcessIndicatorService(final Logger logger, final DeliveryChannel deliveryChannel,
            final ComponentContext context, final SOAParameterType consumerEndpoint,
            final String[] additionalNsAndPref) {
        super();
        this.logger = logger;
        this.dc = deliveryChannel;
        this.cptContext = context;
        this.consumerEndpoint = consumerEndpoint;
        this.additionnalNsAndPrefixMapping = additionalNsAndPref;
        this.returnedSubscriptionRefs = new ConcurrentHashMap<String, EndpointReferenceType>();
    }

    /**
     * Get the returnedSubscriptionRefs
     * 
     * @return the returnedSubscriptionRefs
     */
    public Map<String, EndpointReferenceType> getReturnedSubscriptionRefs() {
        return this.returnedSubscriptionRefs;
    }

    public String subscribeOnStart(final String serviceUnitName, final String topicExprDialect,
            final String topicExprNamespace, final String topicExprPrefix,
            final String topicExprContent, final String msgContentXpathDialect,
            final String msgContentXpath, final String msgContentSOADialect,
            final String msgContentSOAServiceNamespace, final String msgContentSOAServiceName,
            final String msgContentSOAInterfaceNamespace, final String msgContentSOAInterfaceName,
            final String msgContentSOAEndpoint, final InputStream xsltStyleSheet,
            final ActionType actionType, final Boolean contextPolicyCorrelationId,
            final Boolean contextPolicyEndpoint, final Boolean contextPolicyInterface,
            final Boolean contextPolicyMeuuid, final Boolean contextPolicyNotifDate,
            final Boolean contextPolicyService, final Boolean contextPolicyStatus) {

        String resourceUuidToReturn = "";

        try {

            final Subscribe subscribePayload = WSNotificationFactory.getInstance()
            .createSubscribe();
            /*
             * GregorianCalendar gCalendar = new GregorianCalendar();
             * subscribePayload.setInitialTerminationTime(gCalendar.getTime());
             */
            final EndpointReferenceType consumerEdpRef = WSAddressingFactory.getInstance()
            .newEndpointReferenceType();
            consumerEdpRef.setAddress(SOAParameterType.DEFAULT_PETALS_ADDRESS);
            final ReferenceParametersType ref = consumerEdpRef.newReferenceParameters();
            consumerEdpRef.setReferenceParameters(ref);
            WsnSpecificTypeHelper.setSOAParameter(this.consumerEndpoint, ref);

            subscribePayload.setConsumerReference(consumerEdpRef);

            final FilterType filter = WSNotificationFactory.getInstance().createFiltertype();

            final TopicExpressionType topic = WSNotificationFactory.getInstance()
            .createTopicExpressionType();

            topic.addTopicNameSpace(topicExprPrefix, topicExprNamespace);

            topic.setDialect(topicExprDialect);
            topic.setContent(topicExprContent);
            filter.setTopicExpression(topic);

            // MessageContent - Xpath
            if (msgContentXpathDialect != null) {
                final QueryExpressionType msgCtt = WSNotificationFactory.getInstance()
                .createQueryExpressionType();
                msgCtt.setDialect(msgContentXpathDialect);
                // --- add xpath expression ---
                if (msgContentXpath != null) {
                    WsnSpecificTypeHelper.setXpathExpressionsType(msgContentXpath, msgCtt);
                }
                // --- add service expression ---
                // ---------------------------
                filter.addMessageContent(msgCtt);
            }

            // MessageContent - SOAParam
            if (msgContentSOADialect != null) {
                final QueryExpressionType msgCtt = WSNotificationFactory.getInstance()
                .createQueryExpressionType();
                msgCtt.setDialect(msgContentSOADialect);
                // --- add SOA Param

                final SOAParameterType soaVals = WSNotificationExtensionFactory.getInstance()
                .createSOAParameterType();

                if (msgContentSOAServiceName != null) {
                    QName service = null;
                    if (msgContentSOAServiceNamespace != null) {
                        service = new QName(msgContentSOAServiceNamespace, msgContentSOAServiceName);
                    } else {
                        service = new QName(msgContentSOAServiceName);
                    }
                    soaVals.setService(service);
                }

                if (msgContentSOAInterfaceName != null) {
                    QName soaInterface = null;
                    if (msgContentSOAInterfaceNamespace != null) {
                        soaInterface = new QName(msgContentSOAInterfaceNamespace,
                                msgContentSOAInterfaceName);
                    } else {
                        soaInterface = new QName(msgContentSOAInterfaceName);
                    }
                    soaVals.setInterface(soaInterface);
                }

                if (msgContentSOAEndpoint != null) {
                    soaVals.setEndpoint(msgContentSOAEndpoint);
                }

                WsnSpecificTypeHelper.setSOAParameter(soaVals, msgCtt);

                // --- add service expression ---
                // ---------------------------
                filter.addMessageContent(msgCtt);
            }

            // Filter
            subscribePayload.setFilter(filter);

            // ---- Add SubscriptionPolicy obj (xslt stylesheets) ---- :

            final SubscriptionPolicyType policy = WSNotificationFactory.getInstance()
            .createSubscriptionPolicyType();

            if (xsltStyleSheet != null) {
                final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                factory.setNamespaceAware(true);
                final DocumentBuilder builderForXslt = factory.newDocumentBuilder();

                final Document styleSheetAsDoc = builderForXslt.parse(xsltStyleSheet);

                // System.out.println("\n[DEBUG]\t ========== \n[DEBUG] Stylesheet parsed :\n"
                // + XMLPrettyPrinter.prettyPrint(styleSheetAsDoc) +
                // "\n[DEBUG]\t ==========\n");

                final Element styleSheetAsElt = styleSheetAsDoc.getDocumentElement();

                final XsltStyleSheetType transformPolicy = WSNotificationExtensionFactory
                .getInstance().createXsltStyleSheetType();
                transformPolicy.setStyleSheet(styleSheetAsElt);

                WsnSpecificTypeHelper.setXsltStyleSheetType(transformPolicy, policy);
            }

            if (actionType != null) {
                final ProcessPolicyType processPolicy = WSNotificationExtensionFactory
                .getInstance().createProcessPolicyType();
                processPolicy.addAction(actionType);
                WsnSpecificTypeHelper.setProcessPolicyType(processPolicy, policy);
            }

            final ContextPolicyType contextPolicy = WSNotificationExtensionFactory.getInstance()
            .createContextPolicyType();

            contextPolicy.setCorrelationId(contextPolicyCorrelationId);
            contextPolicy.setEndpoint(contextPolicyEndpoint);
            contextPolicy.setInterface(contextPolicyInterface);
            contextPolicy.setMeuuid(contextPolicyMeuuid);
            contextPolicy.setNotifDate(contextPolicyNotifDate);
            contextPolicy.setService(contextPolicyService);
            contextPolicy.setStatus(contextPolicyStatus);

            WsnSpecificTypeHelper.setContextPolicyType(contextPolicy, policy);

            subscribePayload.setSubscriptionPolicy(policy);

            final Document doc = WSNotificationWriter.getInstance(
                    this.additionnalNsAndPrefixMapping).writeSubscribe(subscribePayload);

            // System.out.println("\n[DEBUG]\t ========== \n[DEBUG] Subscibe Payload sent :\n"
            // + XMLPrettyPrinter.prettyPrint(doc) +
            // "\n[DEBUG]\t ==========\n");

            // Send subscription request and store returned subscriptionId
            // SubscribeResponse response =
            // this.wsnReader.readSubscribeResponse((this.subscribe(doc)));
            final Document respAsDoc = this.sendRequest(WsnbConstants.SUBSCRIBE_QNAME,
                    NOTIFICATION_BROKER_SERVICE_QNAME, NOTIFICATION_BROKER_INTERFACE_QNAME,
                    null/* endpoint */, doc);

            SubscribeResponse response = null;

            try {
                response = WSNotificationReader.getInstance().readSubscribeResponse(respAsDoc);
            } catch (final Exception e) {
                this.logger
                .log(
                        Level.WARNING,
                        "An error occurs during \"SubscribeResponse\" unmarshalling process.\n A WS-Notification fault may have been returned as response to the request.");
            }

            if (response != null) {
                final EndpointReferenceType returnedSubsRef = response.getSubscriptionReference();
                final String subscriptionId = WsnSpecificTypeHelper.getResourcesUuidType(
                        returnedSubsRef.getReferenceParameters()).getUuids().get(0);
                this.logger.log(Level.FINE, "subscriptionId returned : " + subscriptionId);

                this.returnedSubscriptionRefs.put(subscriptionId, returnedSubsRef);
                this.persist(returnedSubsRef, serviceUnitName, subscriptionId);

                resourceUuidToReturn = subscriptionId;
            }

            // }
        } catch (final IOException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubscriptions\" proccess");
            // e.printStackTrace();
        } catch (final WSNotificationException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubsciptions\" request proccess");
            // e.printStackTrace();
        } catch (final WSAddressingException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubsciptions\" request proccess");
            // e.printStackTrace();
        } catch (final ParserConfigurationException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubsciptions\" request proccess");
            // e.printStackTrace();
        } catch (final SAXException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubsciptions\" request proccess");
            // e.printStackTrace();
        } catch (final WSNotificationExtensionException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during \"initSubsciptions\" request proccess");
            // e.printStackTrace();
        }

        return resourceUuidToReturn;
    }

    public void unsubscribeOnStop(final String serviceUnitName, final String resourceUuid) {
        if (this.returnedSubscriptionRefs.size() > 0) {
            // Document result = null;
            // convert RegisterPublisher Object to document representation
            try {
                // for (EndpointReferenceType subsRefItem :
                // this.returnedSubscriptionRefs.values()) {

                final EndpointReferenceType subsRefItem = this.returnedSubscriptionRefs
                .get(resourceUuid);

                // create Unsubscribe operation payload
                final Unsubscribe unsubscribePayload = WSNotificationFactory.getInstance()
                .createUnsubscribe();

                final String externalAddress = subsRefItem.getAddress();

                final SOAParameterType itemSOAParam = WsnSpecificTypeHelper
                .getSOAParameter(subsRefItem.getReferenceParameters());

                final String subscriptionManagerEdp = itemSOAParam.getEndpoint();
                final QName subscriptionManagerService = itemSOAParam.getService();
                final QName subscriptionManagerInterface = itemSOAParam.getInterface();

                final EndpointReferenceType edpRef = WSAddressingFactory.getInstance()
                .newEndpointReferenceType();
                edpRef.setAddress(externalAddress);
                WsnSpecificTypeHelper.setEndpointReference(edpRef, unsubscribePayload);

                final String subsRefUuid = WsnSpecificTypeHelper.getResourcesUuidType(
                        subsRefItem.getReferenceParameters()).getUuids().get(0);
                final ResourcesUuidType rUuids = WSNotificationExtensionFactory.getInstance()
                .createResourcesUuidType();
                rUuids.addUuid(subsRefUuid);

                WsnSpecificTypeHelper.setResourcesUuidType(rUuids, unsubscribePayload);

                // convert RegisterPublisher Object to document representation
                final Document requestAsDoc = WSNotificationWriter.getInstance(
                        this.additionnalNsAndPrefixMapping).writeUnsubscribe(unsubscribePayload);
                // result = this.sendRequest(WsnbConstants.UNSUBSCRIBE_QNAME,
                // NOTIFICATION_BROKER_SERVICE_QNAME,
                // NOTIFICATION_BROKER_INTERFACE_QNAME,
                // NOTIFICATION_BROKER_ENDPOINT, requestAsDoc);
                this.sendRequest(WsnbConstants.UNSUBSCRIBE_QNAME, subscriptionManagerService,
                        subscriptionManagerInterface, subscriptionManagerEdp, requestAsDoc);
                // }

                this.returnedSubscriptionRefs.remove(resourceUuid);

                this.removeSubscriptionUuid(serviceUnitName, resourceUuid);

            } catch (final WSNotificationNotImplementedException e) {
                this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                        + "] -An error occurs during \"clearSubsciptions\" request proccess");
                // e.printStackTrace();
            } catch (final WSNotificationException e) {
                this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                        + "] -An error occurs during \"clearSubsciptions\" request proccess");
                // e.printStackTrace();
            } catch (final WSAddressingException e) {
                this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                        + "] -An error occurs during \"clearSubsciptions\" request proccess");
                // e.printStackTrace();
            } catch (final WSNotificationExtensionException e) {
                this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                        + "] -An error occurs during \"clearSubsciptions\" request proccess");
                // e.printStackTrace();
            } catch (final IOException e) {
                this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                        + "] -An error occurs during \"clearSubsciptions\" request proccess");
                // e.printStackTrace();
            }
        }
    }

    private Document sendRequest(final QName operation, final QName qService,
            final QName qInterface, final String endpoint, final Document request) {
        Document result = null;

        InOut inOutExchange = null;
        try {
            // --- case 1 : unsubscribe request
            if (endpoint != null) {
                final ServiceEndpoint serviceEndpoint = this.cptContext.getEndpoint(qService,
                        endpoint);
                if (serviceEndpoint == null) {
                    this.logger.log(Level.FINE, "Endpoint not found for service "
                            + qService.getLocalPart());
                } else {
                    this.logger.log(Level.FINE, "service endpoint registred :"
                            + serviceEndpoint.getEndpointName());
                    inOutExchange = this.dc.createExchangeFactory(serviceEndpoint)
                    .createInOutExchange();
                }
                // --- case 2 : subscribe request or unknown Uuids
            } else if (qInterface.getLocalPart().compareTo(
                    NOTIFICATION_BROKER_INTERFACE_QNAME.getLocalPart()) == 0) {
                inOutExchange = this.dc.createExchangeFactory(qInterface).createInOutExchange();
            } else {
                throw new MessagingException("No endpoint jbi found to preform request '"
                        + operation + "' (Possible cause : bad resource uuid) !");
            }
            if (inOutExchange != null) {
                final NormalizedMessage normalizedMessage = inOutExchange.createMessage();
                normalizedMessage.setContent(new DOMSource(request));
                inOutExchange.setInMessage(normalizedMessage);
                inOutExchange.setOperation(operation);
                if (this.dc.sendSync(inOutExchange)) {
                    if (inOutExchange.getOutMessage() != null) {
                        result = UtilFactory.getSourceUtil().createDocument(
                                inOutExchange.getOutMessage().getContent());
                    } else if (inOutExchange.getFault() != null) {
                        result = UtilFactory.getSourceUtil().createDocument(
                                inOutExchange.getFault().getContent());
                    }
                }
                this.closeExchange(inOutExchange);
            }
        } catch (final MessagingException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during message exchange building and sending proccess");
            // e.printStackTrace();
        } catch (final PEtALSCDKException e) {
            this.logger.log(Level.WARNING, "[" + e.getClass().getSimpleName()
                    + "] -An error occurs during message exchange building and sending proccess");
            // e.printStackTrace();
        }
        return result;
    }

    /**
     * Close the exchange
     * 
     * @param exchange
     *            The exchange to close
     */
    private void closeExchange(final MessageExchange exchange) {
        // close the messageExchange with the JBI NMR
        try {
            if (ExchangeStatus.ACTIVE.equals(exchange.getStatus())) {
                exchange.setStatus(ExchangeStatus.DONE);
                this.dc.send(exchange);
            }
        } catch (final MessagingException e) {
            final String errorMsg = "Error while closing JBI MessageExchange.";
            this.logger.log(Level.WARNING, errorMsg, e);
        }
    }

    // -------------------------------------------------------
    // ----------- Methods for Persistence feature -----------

    /**
     * Init Persistance folder for subscriptions uuid received by the
     * "KPI Service Engine". Create the folder if it has not already been
     * created.
     */
    public void init() {
        this.receivedUuidParentFolder = new File(this.cptContext.getWorkspaceRoot() + File.separatorChar
                + "SaveReceivedSubscriptionUuid" + File.separatorChar);
        if (this.receivedUuidParentFolder == null) {
            throw new NullPointerException("Persistence folder for kpi component must not be null");
        }
        if (!this.receivedUuidParentFolder.exists()) {
            if (!this.receivedUuidParentFolder.mkdirs()) {
                if (this.logger != null) {
                    this.logger.warning("unable to create the receive uuid folder");
                }
            }
        }
    }
    
    // ----- TDEJEAN : Multi threaded Persistence feature version -----
    // ----- NOTE : not tested yet !!!!
    /*
     * Init Persistance folder for subscriptions uuid received by the
     * "KPI Service Engine". Create the folder if it has not already been
     * created. 
     * tdejean : used for multi-threaded version
     */
    public void init(String serviceUnitName) {
        this.receivedUuidParentFolder = new File(this.cptContext.getWorkspaceRoot() + File.separatorChar
                + "SaveReceivedSubscriptionUuid" + File.separatorChar);
        
        File receivedUuidSUFolder = new File(this.receivedUuidParentFolder.getAbsolutePath() + File.separatorChar + serviceUnitName + File.separatorChar);
        
        if (receivedUuidSUFolder == null) {
            throw new NullPointerException("Persistence folder for kpi component must not be null");
        }
        if (!receivedUuidSUFolder.exists()) {
            if (!receivedUuidSUFolder.mkdirs()) {
                if (this.logger != null) {
                    this.logger.warning("unable to create the receive uuid SU folder for service Unit : \"" + serviceUnitName + "\"");
                }
            }
        }
    }
    // -------------------------------------- //
    
    /**
     * persist a subscription uuid received in a subfolder whose name his the
     * service unit one
     * 
     * @param SUName
     *            the service unit name
     * @param subscriptionUuid
     *            the uuid to store (as an empty file whose name is the uuid)
     * @throws IOException
     * @throws WSAddressingException
     */

    private void persist(final EndpointReferenceType subscriptionRef, final String SUName,
            final String subscriptionUuid) throws IOException, WSAddressingException {

        final File serviceUnitPersistenceFolder = new File(this.receivedUuidParentFolder
                .getAbsolutePath()
                + File.separatorChar + SUName + File.separatorChar);

        if (!serviceUnitPersistenceFolder.exists()) {
         
            // ----- TDEJEAN : Multi threaded Persistence feature version -----
            // ----- NOTE : not tested yet !!!!
            
            // -- original code :
            
            /*if (!serviceUnitPersistenceFolder.mkdirs()) {
                if (this.logger != null) {
                    this.logger.warning("unable to create the service unit persistence folder");
                }
            }*/
            
            // -- new code :
                        
            if (this.logger != null) {
                this.logger.warning("service unit persistence folder not found !\n" +
                		"It must have been initialized during \"on deploy\" Service unit set)");
            }

            // ----------------------------------------------
        }
        final File persist = new File(serviceUnitPersistenceFolder, subscriptionUuid);
        if (persist == null) {
            throw new IOException("The persistance file can not be created");
        }
        if (!persist.exists()) {
            if (!persist.createNewFile()) {
                if (this.logger != null && this.logger.isLoggable(Level.FINE)) {
                    this.logger.fine("unable to create the persist folder");
                }
            }
        }
        WSAddressingFactory.getInstance().newWSAddressingWriter().writeEndpointReferenceType(
                subscriptionRef, persist);
    }

    /**
     * remove a given subscription uuid according to a given Service Unit name
     * 
     * @param SUName
     * @param uuid
     * @throws IOException
     */
    private void removeSubscriptionUuid(final String SUName, final String uuid) throws IOException {

        final File subscriptionToRemove = new File(this.receivedUuidParentFolder.getAbsolutePath()
                + File.separatorChar + SUName + File.separatorChar, uuid);

        if (subscriptionToRemove.exists()) {
            if (!subscriptionToRemove.delete()) {
                throw new IOException("Deletion of subscription uuid \"" + uuid + "\" failed !");
            }
        }
    }

    /**
     * 
     * @return
     * @throws WSNotificationException
     * @throws WSAddressingException
     */
    public List<String> restoreSubscriptionRef(final String SUName) throws WSNotificationException,
    WSAddressingException {

        List<String> restoredSubsUuids = null;

        final File serviceUnitPersistenceFolder = new File(this.receivedUuidParentFolder
                .getAbsolutePath()
                + File.separatorChar + SUName + File.separatorChar);

        if (serviceUnitPersistenceFolder.exists()) {

            restoredSubsUuids = new ArrayList<String>();

            final File[] persistedSubsriptionFiles = serviceUnitPersistenceFolder.listFiles();
            String uuid;
            for (final File persistedSubsriptionFile : persistedSubsriptionFiles) {
                uuid = persistedSubsriptionFile.getName();
                this.returnedSubscriptionRefs.put(uuid, WSAddressingFactory.getInstance()
                        .newWSAddressingReader()
                        .readEndpointReferenceType(persistedSubsriptionFile));
                restoredSubsUuids.add(uuid);
            }
        }
        return restoredSubsUuids;
    }

    /**
     * return the uuid previously stored for a given service unit name
     * 
     * @param SUName
     *            the service unit name
     * @return list of uuid (String)
     */
    public List<String> getSubscriptionUuidToRestore(final String SUName) {

        final List<String> result = new ArrayList<String>();

        final File serviceUnitPersistFolder = new File(this.receivedUuidParentFolder.getAbsolutePath()
                + File.separatorChar + SUName + File.separatorChar);

        if (serviceUnitPersistFolder.exists()) {
            final File[] persistedSubsriptionFiles = serviceUnitPersistFolder.listFiles();
            for (final File persistedSubsriptionFile : persistedSubsriptionFiles) {
                result.add(persistedSubsriptionFile.getName());
            }
        }
        return result;
    }
    // -------------------------------------------------------

}
