/**
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $id.java
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.engine.kpi.su;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.InterfaceType;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;
import org.ow2.petals.engine.kpi.KeyProcessIndicatorService;

import com.ebmwebsourcing.wsstar.addressing.definition.api.WSAddressingException;
import com.ebmwebsourcing.wsstar.notification.definition.utils.WSNotificationException;
import com.ebmwebsourcing.wsstar.notification.extension.WSNotificationExtensionFactory;
import com.ebmwebsourcing.wsstar.notification.extension.api.SOAParameterType;
import com.ebmwebsourcing.wsstar.notification.extension.utils.WSNotificationExtensionException;
import com.ebmwebsourcing.wsstar.notification.jaxbextension.types.ActionType;

public class KpiServiceUnitManager extends AbstractServiceUnitManager {

    private final Map<String, List<String>> SUSubscriptions;

    private final Map<String, String> SURootPaths;

    private static final String NOTIFICATION_CONSUMER_SERVICE_NAME = "KeyProcessIndicatorService";

    private KeyProcessIndicatorService keyProcessIndicatorService = null;

    public KpiServiceUnitManager(final AbstractComponent component) {
        super(component);
        this.SUSubscriptions = new ConcurrentHashMap<String, List<String>>();
        this.SURootPaths = new ConcurrentHashMap<String, String>();
    }

    /**
     * Get the keyProcessIndicatorService
     * 
     * @return the keyProcessIndicatorService
     */
    public KeyProcessIndicatorService getKeyProcessIndicatorService() {
        return this.keyProcessIndicatorService;
    }

    @Override
    protected void doDeploy(final String serviceUnitName, final String suRootPath,
            final Jbi jbiDescriptor) throws PEtALSCDKException {
        super.doDeploy(serviceUnitName, suRootPath, jbiDescriptor);
        this.SURootPaths.put(serviceUnitName, suRootPath);
        
        // --- TDEJEAN : Multi threaded version of Persistence feature
        // -- original code :
        //this.keyProcessIndicatorService.init();
        // -- new code :
        this.keyProcessIndicatorService.init(serviceUnitName);
        // ------------------------
    }

    @Override
    protected void doStart(final String serviceUnitName) throws PEtALSCDKException {
        super.doStart(serviceUnitName);
        List<String> subscriptionUuidPersisted = null;
        try {
            subscriptionUuidPersisted = this.keyProcessIndicatorService
            .restoreSubscriptionRef(serviceUnitName);
        } catch (final WSNotificationException e1) {
            e1.printStackTrace();
        } catch (final WSAddressingException e1) {
            e1.printStackTrace();
        }

        if (subscriptionUuidPersisted != null && subscriptionUuidPersisted.size() > 0) {
            this.SUSubscriptions.put(serviceUnitName, subscriptionUuidPersisted);
        } else {

            final List<String> receivedUuid = new ArrayList<String>();

            final List<Consumes> consumesList = this.getServiceUnitDataHandlers().get(
                    serviceUnitName).getDescriptor().getServices().getConsumes();

            String topicExprDialect = "", topicExprNamespace = "", topicExprPrefix = "", topicExprContent = "", msgContentXpathDialect = "", msgContentXpath = "", msgContentSOADialect = "", msgContentSOAServiceNamespace = "", msgContentSOAServiceName = "", msgContentSOAInterfaceNamespace = "", msgContentSOAInterfaceName = "", msgContentSOAEndpoint = "", xsltStylesheetPath = "";

            Boolean contextPolicyCorrelationId = null, contextPolicyInterface = null, contextPolicyEndpoint= null, contextPolicyMeuuid= null, contextPolicyNotifDate= null, contextPolicyService= null, contextPolicyStatus= null;

            InputStream xsltStyleSheet = null;
            ActionType actionType = null;

            for (final Consumes consumeItem : consumesList) {
                final ConfigurationExtensions extensions = new ConfigurationExtensions(consumeItem.getAny());
                topicExprDialect = extensions.get("TopicExpressionDialect");
                topicExprNamespace = extensions.get("TopicExpressionNamespace");
                topicExprPrefix = extensions.get("TopicExpressionPrefix");
                topicExprContent = extensions.get("TopicExpressionContent");
                msgContentXpathDialect = extensions.get("MessageContentXpathDialect");
                msgContentXpath = extensions.get("MessageContentXpath");

                msgContentSOADialect = extensions.get("MessageContentSOADialect");
                msgContentSOAServiceNamespace = extensions.get("MessageContentSOAServiceNamespace");
                msgContentSOAServiceName = extensions.get("MessageContentSOAServiceName");
                msgContentSOAInterfaceNamespace = extensions
                .get("MessageContentSOAInterfaceNamespace");
                msgContentSOAInterfaceName = extensions.get("MessageContentSOAInterfaceName");
                msgContentSOAEndpoint = extensions.get("MessageContentSOAEndpoint");

                xsltStylesheetPath = extensions.get("TransformPolicy");
                if (xsltStylesheetPath != null)
                    try {
                        xsltStyleSheet = new FileInputStream(this.SURootPaths.get(serviceUnitName)
                                + xsltStylesheetPath);
                    } catch (final FileNotFoundException e) {
                        e.printStackTrace();
                    }

                if (extensions.get("ProcessPolicyAction") != null)    
                    actionType = ActionType.fromValue(extensions.get("ProcessPolicyAction"));

                if (extensions.get("ContextPolicyCorrelationId") != null)
                    contextPolicyCorrelationId = Boolean.getBoolean(extensions
                        .get("ContextPolicyCorrelationId"));
                
                if (extensions.get("ContextPolicyEndpoint") != null)
                    contextPolicyEndpoint = Boolean.getBoolean(extensions.get("ContextPolicyEndpoint"));
                
                if (extensions.get("ContextPolicyInterface") != null)
                    contextPolicyInterface = Boolean.getBoolean(extensions
                        .get("ContextPolicyInterface"));
                
                if (extensions.get("ContextPolicyMeuuid") != null)
                    contextPolicyMeuuid = Boolean.getBoolean(extensions.get("ContextPolicyMeuuid"));
               
                if (extensions.get("ContextPolicyNotifDate") != null)
                    contextPolicyNotifDate = Boolean.getBoolean(extensions.get("ContextPolicyNotifDate"));
                
                if (extensions.get("ContextPolicyService") != null)
                    contextPolicyService = Boolean.getBoolean(extensions.get("ContextPolicyService"));
                
                if (extensions.get("ContextPolicyStatus") != null)
                    contextPolicyStatus = Boolean.getBoolean(extensions.get("ContextPolicyStatus"));

                final String ruuid = this.keyProcessIndicatorService.subscribeOnStart(
                        serviceUnitName, topicExprDialect, topicExprNamespace, topicExprPrefix,
                        topicExprContent, msgContentXpathDialect, msgContentXpath,
                        msgContentSOADialect, msgContentSOAServiceNamespace,
                        msgContentSOAServiceName, msgContentSOAInterfaceNamespace,
                        msgContentSOAInterfaceName, msgContentSOAEndpoint, xsltStyleSheet,
                        actionType, contextPolicyCorrelationId, contextPolicyEndpoint,
                        contextPolicyInterface, contextPolicyMeuuid, contextPolicyNotifDate,
                        contextPolicyService, contextPolicyStatus);
                if (ruuid.length() > 0) {
                    receivedUuid.add(ruuid);
                }
            }
            this.SUSubscriptions.put(serviceUnitName, receivedUuid);
        }
    }

    @Override
    protected void doStop(final String serviceUnitName) throws PEtALSCDKException {

        final List<String> uuidList = this.SUSubscriptions.get(serviceUnitName);
        if (uuidList != null) {
            for (final String uuidItem : uuidList) {
                this.keyProcessIndicatorService.unsubscribeOnStop(serviceUnitName, uuidItem);
            }
            this.SUSubscriptions.remove(serviceUnitName);
        }
        super.doStop(serviceUnitName);
    }

    @Override
    protected void doUndeploy(final String serviceUnitName) throws PEtALSCDKException {
        // TODO Auto-generated method stub
        this.SURootPaths.remove(serviceUnitName);
        super.doUndeploy(serviceUnitName);
    }

    public void createKeyProcessIndicatorService() {
        try {
            final String[] wsdlNsPrefPairs = this.component.getNativeWsdl().getDescription()
            .getNamespaces().getPreDeclaredNamespaceUris2();
            final ConfigurationExtensions componentConfig = new ConfigurationExtensions(
                    this.component.getComponentConfiguration().getAny());
            final String hibernateConfigKey = "hibernate.default.config";
            if (componentConfig.containsKey("hibernateURL")) {
                final String hibernateURL = componentConfig.get("hibernateURL");
                if (hibernateURL != null && hibernateURL.length() != 0) {
                    final URL hibernateConfigURL = new URL(hibernateURL);
                    System.setProperty(hibernateConfigKey, hibernateConfigURL.toExternalForm());
                }
            }
            final QName kpiServiceQName = new QName(this.component.getNativeWsdl().getDescription()
                    .getTargetNamespace(), NOTIFICATION_CONSUMER_SERVICE_NAME);
            final InterfaceType kpiIntf = this.component.getNativeWsdl().getDescription()
            .getService(kpiServiceQName).getInterface();
            final Endpoint kpiComponentEdpt = this.component.getNativeWsdl().getDescription()
            .findEndpointsImplementingInterface(kpiIntf).get(0);
            final SOAParameterType soaParam = WSNotificationExtensionFactory.getInstance()
            .createSOAParameterType();
            soaParam.setService(kpiServiceQName);
            soaParam.setInterface(kpiIntf.getQName());
            soaParam.setEndpoint(kpiComponentEdpt.getName());
            this.keyProcessIndicatorService = new KeyProcessIndicatorService(this.logger,
                    this.component.getChannel(), this.component.getContext(), soaParam,
                    wsdlNsPrefPairs);
        } catch (final MalformedURLException e) {
            this.logger.warning("unable to init the listener");
            this.logger.fine("detail on error: " + e.getMessage());
        } catch (final WSDLException e) {
            this.logger.warning("unable to init the listener");
            this.logger.fine("detail on error: " + e.getMessage());
        } catch (final WSNotificationExtensionException e) {
            this.logger.warning("unable to init the listener");
            this.logger.fine("detail on error: " + e.getMessage());
        }
    }
}
